#include "md3import.h"

void adjust_bounds(q3_frame &frame, q3_vertex &vert)
{
	float coords[3];

	coords[0] = (float) vert.x * MD3_XYZ_SCALE;
	coords[1] = (float) vert.y * MD3_XYZ_SCALE;
	coords[2] = (float) vert.z * MD3_XYZ_SCALE;

	if(frame.mins[0] > frame.maxs[0]) //these being backwards tells us no vertices have been processed for this frame
	{
		frame.mins[0] = frame.maxs[0] = coords[0];
		frame.mins[1] = frame.maxs[1] = coords[1];
		frame.mins[2] = frame.maxs[2] = coords[2];
		return;
	}

	if(frame.mins[0] > coords[0])
		frame.mins[0] = coords[0];
	if(frame.mins[1] > coords[1])
		frame.mins[1] = coords[1];
	if(frame.mins[2] > coords[2])
		frame.mins[2] = coords[2];

	if(frame.maxs[0] < coords[0])
		frame.maxs[0] = coords[0];
	if(frame.maxs[1] < coords[1])
		frame.maxs[1] = coords[1];
	if(frame.maxs[2] < coords[2])
		frame.maxs[2] = coords[2];
}

void parse_surface(std::ifstream &q3input, q3_surface& surf, q3_model &q3mdl)
{
	// triangles
	q3input.seekg(surf.filepoint + surf.header.ofs_triangles);
	for(int32_t i = 0; i < surf.header.num_triangles; i++)
	{
		q3_triangle tri;
		q3input.read((char*)&tri, sizeof(q3_triangle));
		surf.triangle.push_back(tri);
	}

	// vertices - is there a need to match up these with the triangle indices explicitly?
	q3input.seekg(surf.filepoint + surf.header.ofs_vertices);
	surf.frame.resize(surf.header.num_frames);

	for(int32_t i = 0; i < surf.header.num_frames; i++)
	{
		for(int32_t j = 0; j < surf.header.num_verts; j++)
		{
			q3_vertex vert;
			q3input.read((char*)&vert, sizeof(q3_vertex));
			surf.frame[i].vertex.push_back(vert); //add it to the last element of surf.frame
			adjust_bounds( q3mdl.frames[i],surf.frame[i].vertex[j]);
		}
	}

	// uv map
	q3input.seekg(surf.filepoint + surf.header.ofs_st);
	for(int32_t i = 0; i < surf.header.num_verts; i++)
	{
		q3_texcoord uv;
		q3input.read((char*)&uv, sizeof(q3_texcoord));
		surf.texcoord.push_back(uv);
	}
}


int32_t parse_q3model(std::ifstream &q3input, q3_model &q3mdl)
{
	//Check for correct file format
	q3input.seekg( 0, std::ios::beg );
	q3mdl.header.ident[0] = 0;//ensure that random data doesn't ever match the header
	q3input.read((char*)&q3mdl.header, sizeof(q3_header));
	if ( strncmp((char*)q3mdl.header.ident,"IDP3",4) )
	{
		std::cout << "Input is not a valid md3 model, ident value:" << q3mdl.header.ident << std::endl;
		return 0;
	}
	std::cout << "model is version " << q3mdl.header.version << std::endl;
	std::cout << "model frames: " << q3mdl.header.framecount << std::endl;
	std::cout << "model surfaces: " << q3mdl.header.surfacecount << std::endl;

	// load the frames
	q3input.seekg(q3mdl.header.ofs_frames);
	for( int32_t i = 0; i < q3mdl.header.framecount; i++)
	{
		q3_frame currentframe;
		q3input.read((char*)&currentframe, sizeof(q3_frame));
		currentframe.mins[0] = 1;
		currentframe.maxs[0] = -1;

		q3mdl.frames.push_back(currentframe);
	}

	// load the surface headers
	q3input.seekg(q3mdl.header.ofs_surfaces);
	for( int32_t i = 0; i < q3mdl.header.surfacecount; i++)
	{
		int headpoint;
		// this cast suppresses a warning about loss of data
		// the MD3 format doesn't support 64 bit file offsets
		// I don't think an MD3 files should ever exceed 4GB though...
		headpoint = static_cast<int>(q3input.tellg());
		q3_surfaceheader currentsurfheader;
		q3_surface currentsurf;
		q3input.read((char*)&currentsurfheader, sizeof(q3_surfaceheader));
		currentsurf.header = currentsurfheader;
		currentsurf.filepoint = headpoint;
		q3mdl.surfaces.push_back(currentsurf);
		q3input.seekg(currentsurf.filepoint + currentsurf.header.ofs_end ); // skip to start of next header
	}

	//now actually load the contents of each surface
	std::vector<q3_surface>::iterator surfiter;
	for (surfiter = q3mdl.surfaces.begin(); surfiter != q3mdl.surfaces.end(); surfiter++)
	{
		parse_surface(q3input, *surfiter, q3mdl);
	}
	return 1;
}
